#ifndef TexturesH //To avoid linker errors, define header's unique "name" so...
#define TexturesH //...this header will not be compilled more than one time
//==============================================================================
#include <gl\gl.h>      // Header file for the main OpenGL library
#include <gl\glu.h>     // Header file for the GLu library
#include <stdlib.h>     // Standart c library
#include <stdio.h>      // File I/O routines

//------------------------------------------------------------------------------

//Auxilary micro-function for rotating red and blue bytes
inline void BGRtoRGB(unsigned char *BGR){
 unsigned char B = *BGR;
 BGR[0]=BGR[2];
 BGR[2]=B;
}

//Class for managing basic 2D texture
class TTexture{
//Symbolic texture name, loaded into graphic memory
unsigned int Name;
public:
//Default constructor and destructor
inline TTexture():Name(0){}
inline ~TTexture(){if(Name)glDeleteTextures(1,&Name);}
//Function converts 24bit bitmap (*.bmp) to TEX file format (*.tex);
inline bool BMPtoTEX(const char* FilePath);

//!!!NOTE!!!
//all of the following texture routines can be called only after GL have been
//initialized!!!

//Function loads texture image from windows bitmap file and registers it;
//note that only 24bit-depth bitmaps supported!
inline bool LoadFromBMP(const char* FilePath);
//Function loads texture from TEX file; it's format extremely simple:
// 4 byte:   unsigned int, width of the image in pixels
// 4 byte:   unsigned int, height of the image in pixels
//(4*width*height) bytes:   pixels data; image' pixels stored row-by-row, each
//                          one in RGBA format (4 unsigned chars per pixel);
inline bool LoadFromTEX(const char* FilePath);
//Function loads texture from currently opened file starting to read it from
//current file read/write position and registers texture; after loading done,
//file read/write position will be shifted to the end of texture chunk.
//Such texture chunk must have the folowing format:
// 4 byte:   unsigned int, width of the image in pixels
// 4 byte:   unsigned int, height of the image in pixels
//(4*width*height) bytes:   pixels data; image' pixels stored row-by-row, each
//                          one in RGBA format (4 unsigned chars per pixel);
inline void LoadFromChunk(FILE* CurrentlyOpenedFile);
//Function just binds this texture to current 2D-texture unit
inline void Bind(){glBindTexture(GL_TEXTURE_2D,Name);}
};

//++++++++++++++++++++++++ TTexture class  definition ++++++++++++++++++++++++\\
inline bool TTexture::BMPtoTEX(const char* FilePath){
 //Try to open specyfied file; if usuccesfull, return negative result
 FILE* FileHandler=fopen(FilePath,"rb"); if(!FileHandler)return false;
 //Pass 18 bytes and read width and height of the image
 fseek(FileHandler,18,0);
 unsigned long int wh[2]; fread(wh,4,2,FileHandler);
 //Pass two bytes and read bpp
 fseek(FileHandler,2,1);
 unsigned short int bpp;  fread(&bpp,2,1,FileHandler);
 //We handle only RGB (24bpp) bitmap types, so check it
 if(bpp!=24){ fclose(FileHandler); return false; }
 //Pass 24 bytes of stupid header's garbage and read pixels
 fseek(FileHandler,24,1);
 //Count size of row (don't forget about 4-byte-aligning!)
 unsigned long int rowsize = wh[0]*3 + wh[0]%4;
 //Calculate total size of image data
 unsigned long int size = wh[1]*rowsize;
 //Create storage for pixels and load them into it from file
 unsigned char *pixels = new unsigned char[size];
 fread(pixels,size,1,FileHandler);
 //Close bmp file
 fclose(FileHandler);
 //Genenrate new file name
 unsigned int PathLength = strlen(FilePath);
 char* NewName = new char[PathLength+1];
 strcpy(NewName,FilePath);
 NewName[PathLength-3]='t';NewName[PathLength-2]='e'; NewName[PathLength-1]='x';
 //Create new file in the same path with the same name, but with .TEX extension
 FileHandler=fopen(NewName,"wb");if(!FileHandler){delete[] pixels;return false;}
 //Write image dimensions
 fwrite(wh,4,2,FileHandler);
 //Write all pixels, converting them from BGR to RGBA representation first
 char p[4]={0,0,0,255};
 for(int h=0;h<wh[1];h++)for(int w=0;w<wh[0];w++){
  p[0] = pixels[h*rowsize+w*3+2];
  p[1] = pixels[h*rowsize+w*3+1];
  p[2] = pixels[h*rowsize+w*3];
  fwrite(p,1,4,FileHandler);
 }
 //Close file and destroy pixels location
 fclose(FileHandler);
 delete[] pixels;
 //As no problems occured, return positive result
 return true;
}
//------------------------------------------------------------------------------
inline bool TTexture::LoadFromBMP(const char* FilePath){
 //Try to open specyfied file; if usuccesfull, return negative result
 FILE* FileHandler=fopen(FilePath,"rb"); if(!FileHandler)return false;
 //Pass 18 bytes and read width and height of the image
 fseek(FileHandler,18,0);
 unsigned long int wh[2]; fread(wh,4,2,FileHandler);
 //Pass two bytes and read bpp
 fseek(FileHandler,2,1);
 unsigned short int bpp;  fread(&bpp,2,1,FileHandler);
 //We handle only RGB (24bpp) bitmap types, so check it
 if(bpp!=24){ fclose(FileHandler); return false; }
 //Pass 24 bytes of stupid header's garbage and read pixels
 fseek(FileHandler,24,1);
 //Count size of row (don't forget about 4-byte-aligning!)
 unsigned long int rowsize = wh[0]*3 + wh[0]%4;
 //Calculate total size of image data
 unsigned long int size = wh[1]*rowsize;
 //Create storage for pixels and load them into it from file
 unsigned char *pixels = new unsigned char[size];
 fread(pixels,size,1,FileHandler);
 //Swap Red and Green bytes
 for(int h=0;h<wh[1];h++)for(int w=0;w<wh[0];w++)
  BGRtoRGB(&pixels[h*rowsize+w*3]);
 //Create, load, register texure
 glGenTextures(1,&Name);            //Create new texture name
 glBindTexture(GL_TEXTURE_2D,Name); //Make it current
 //Create mipmaps
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGB,
                   wh[0],wh[1],
                   GL_RGB,GL_UNSIGNED_BYTE,pixels);
 //Close file and destroy pixels location
 fclose(FileHandler);
 delete[] pixels;
 //As no problems occured, return positive result
 return true;
}
//------------------------------------------------------------------------------
inline bool TTexture::LoadFromTEX(const char* FilePath){
 //Try to open specyfied file; if usuccesfull, return negative result
 FILE* FileHandler=fopen(FilePath,"rb"); if(!FileHandler)return false;
 //Read image dimensions
 unsigned long int wh[2]; fread(wh,4,2,FileHandler);
 //Calculate total size of image data
 unsigned long int size = wh[0]*wh[1]*4;
 //Create storage for pixels and load them into it from file
 unsigned char *pixels = new unsigned char[size];
 fread(pixels,size,1,FileHandler);
 //Create, load, register texure
 glGenTextures(1,&Name);            //Create new texture name
 glBindTexture(GL_TEXTURE_2D,Name); //Make it current
 //Create mipmaps
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,
                   wh[0],wh[1],
                   GL_RGBA,GL_UNSIGNED_BYTE,pixels);
 //Close file and destroy pixels location
 fclose(FileHandler);
 delete[] pixels;
 //As no problems occured, return positive result
 return true;
}
//------------------------------------------------------------------------------
inline void TTexture::LoadFromChunk(FILE* CurrentlyOpenedFile){
 //Read image dimensions
 unsigned long int wh[2]; fread(wh,4,2,CurrentlyOpenedFile);
 //Calculate total size of image data
 unsigned long int size = wh[0]*wh[1]*4;
 //Create storage for pixels and load them into it from file
 unsigned char *pixels = new unsigned char[size];
 fread(pixels,size,1,CurrentlyOpenedFile);
 //Create, load, register texure
 glGenTextures(1,&Name);            //Create new texture name
 glBindTexture(GL_TEXTURE_2D,Name); //Make it current
 //Create mipmaps
 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,
                   wh[0],wh[1],
                   GL_RGBA,GL_UNSIGNED_BYTE,pixels);
 //Destroy pixels location
 delete[] pixels;
}
//------------------------------------------------------------------------------

//==============================================================================
#endif //End of header
